'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = function (_ref) {
  var t = _ref.types;

  var TYPES = /(controller|config|service|filter|animation|provider|directive|factory|run|component)/;

  var presets = {
    'angular': [['$injector.invoke', ['$injectFunction']], ['$httpProvider.interceptors.push', ['$injectFunction']]],
    'ngRoute': [['$routeProvider.when', ['_', {
      'controller': '$injectFunction',
      'resolve': '$injectObject'
    }]]],
    'ui.router': [['$stateProvider.state', ['_', {
      'resolve': '$injectObject',
      'controller': '$injectFunction',
      'onEnter': '$injectFunction',
      'onExit': '$injectFunction'
    }]]],
    'ngMaterial': [['$mdDialog.show', [{
      'controller': '$injectFunction'
    }]], ['$mdToast.show', [{
      'controller': '$injectFunction'
    }]], ['$mdBottomSheet.show', [{
      'controller': '$injectFunction'
    }]]]
  };

  var configuration = void 0;

  function normalizeConfig(config) {
    return config.reduce(function (acc, entry) {
      if (typeof entry === 'string') {
        if (!(entry in presets)) {
          throw new Error('Cannot find preset named \'' + entry + '\'');
        }
        return acc.concat(presets[entry]);
      } else {
        acc.push(entry);
        return acc;
      }
    }, []);
  }

  function annotateUsingConfiguration(memberExprPath) {
    var config = normalizeConfig(configuration);
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = config[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var entry = _step.value;

        var _entry = _slicedToArray(entry, 2),
            memberExpr = _entry[0],
            params = _entry[1];

        var memberExprParts = memberExpr.split('.');
        var identifier = memberExprParts[0];
        var methods = memberExprParts.slice(1);

        if (matchesPattern(memberExprPath, identifier, methods[0])) {
          var currentMemberPath = memberExprPath;
          methods = methods.slice(1);

          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = methods[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var method = _step2.value;

              currentMemberPath = currentMemberPath.parentPath;
              if (!currentMemberPath.get('property').isIdentifier({ name: method })) {
                return;
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }

          var _loop = function _loop(index) {
            var param = params[index];
            var paramPath = currentMemberPath.parentPath.get('arguments')[index];

            if (typeof param === 'string') {
              if (param === '$injectFunction') {
                annotateFunction(paramPath);
              } else if (param === '$injectObject') {
                annotateObjectProperties(paramPath);
              }
            } else if ((typeof param === 'undefined' ? 'undefined' : _typeof(param)) === 'object' && param !== null) {
              eachObjectProperty(paramPath, function (property) {
                // eslint-disable-line no-loop-func
                if (property.get('key').isIdentifier()) {
                  var strategy = param[property.node.key.name];
                  if (strategy === '$injectFunction') {
                    annotateFunction(property.get('value'));
                  } else if (strategy === '$injectObject') {
                    annotateObjectProperties(property.get('value'));
                  }
                }
              });
            }
          };

          for (var index = 0; index < params.length; index++) {
            _loop(index);
          }
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }
  }

  function findRootDeclarator(path) {
    if (path.isIdentifier()) {
      var binding = path.scope.getBinding(path.node.name);

      if (binding) {
        var bpath = binding.path;

        if (bpath.isVariableDeclarator()) {
          if (bpath.get('init').isIdentifier()) {
            return findRootDeclarator(bpath.get('init'));
          } else {
            return bpath;
          }
        }

        if (bpath.isFunction() || bpath.isClassDeclaration() || bpath.isIdentifier()) {
          return bpath;
        }
      }
    }
  }

  function matchesPattern(memberExprPath, identifierName, methodName) {
    if (!memberExprPath.get('property').isIdentifier({ name: methodName })) {
      return false;
    }
    var object = memberExprPath.get('object');

    // resolve chained calls
    while (object.isCallExpression()) {
      if (object.get('callee').isMemberExpression()) {
        object = object.get('callee.object');
      }
    }

    var declarator = findRootDeclarator(object);
    return declarator && declarator.isIdentifier({ name: identifierName });
  }

  function annotateProvide(memberExprPath) {
    var types = ['decorator', 'service', 'factory', 'provider'];
    var matchedType = void 0;

    function matchesProvide(type) {
      if (matchesPattern(memberExprPath, '$provide', type)) {
        matchedType = type;
        return true;
      }
    }

    if (types.some(matchesProvide)) {
      var func = last(memberExprPath.parentPath.get('arguments'));
      annotateFunction(func);
      if (matchedType === 'provider') {
        func.traverse(providerGetVisitor);
      }
    }
  }

  var defaultVisitor = {
    MemberExpression: function MemberExpression(path) {
      annotateUsingConfiguration(path);
    }
  };

  var configVisitor = {
    MemberExpression: function MemberExpression(path) {
      annotateUsingConfiguration(path);
      annotateProvide(path);
    }
  };

  var directiveControllerVisitor = {
    Property: function Property(path) {
      if (path.get('key').isIdentifier({ name: 'controller' })) {
        annotateFunction(path.get('value'));
        path.stop();
      }
    }
  };

  var providerGetVisitor = {
    ExpressionStatement: function ExpressionStatement(path) {
      if (path.get('expression.left').matchesPattern('this.$get')) {
        annotateFunction(path.get('expression.right'));
        path.stop();
      }
    }
  };

  function isAngularModule(path) {
    if (!path.isCallExpression()) {
      return false;
    }

    while (path.get('callee').isMemberExpression() && !path.get('callee').matchesPattern('angular.module')) {
      path = path.get('callee.object');
    }

    if (path.isIdentifier()) {
      var declarator = findRootDeclarator(path);
      if (declarator && declarator.get('init')) {
        path = declarator.get('init');
        return isAngularModule(path);
      }
    }

    if (path.get('callee').matchesPattern('angular.module')) {
      annotateModuleConfigFunction(path.get('callee'));
      return true;
    }

    return false;
  }

  function getTypeName(path) {
    if (path.isCallExpression() && path.get('callee').isMemberExpression()) {
      return path.node.callee.property.name;
    }
  }

  function getFunctionExpressionFromConstructor(classPath) {
    var functionExpression = void 0;
    var visitor = {
      ClassMethod: function ClassMethod(path) {
        if (path.node.kind === 'constructor') {
          functionExpression = path;
          path.stop();
        }
      }
    };
    classPath.traverse(visitor);
    return functionExpression;
  }

  function annotateFunctionImpl(func, original) {
    if (!func) {
      return;
    }

    if (func.isFunction()) {
      var varLiterals = func.node.params.map(function (i) {
        return t.stringLiteral(i.name);
      });
      varLiterals.push(original.node);
      original.replaceWith(t.arrayExpression(varLiterals));
    }

    if (func.isClass()) {
      var node = func.node;
      var siblings = func.parent.body;

      var has$injectStaticProperty = false;

      if (siblings) {
        var nextSiblings = siblings.slice(siblings.indexOf(node) + 1);

        var staticProperties = nextSiblings.filter(function (it) {
          return it.type === 'ExpressionStatement' && it.expression.type === 'AssignmentExpression' && it.expression.operator === '=' && it.expression.left.object.name === node.id.name;
        });

        has$injectStaticProperty = staticProperties.some(function (it) {
          return it.expression.left.property.name === '$inject';
        });
      }

      if (!has$injectStaticProperty) {
        var functionExpression = getFunctionExpressionFromConstructor(func);
        annotateFunctionImpl(functionExpression, original);
      }
    }

    if (func.isIdentifier()) {
      var declarator = findRootDeclarator(func);
      if (declarator) {
        if (declarator.isVariableDeclarator()) {
          annotateFunctionImpl(declarator.get('init'), original);
        } else {
          annotateFunctionImpl(declarator, original);
        }
      }
    }
  }

  function eachObjectProperty(objectOrIdentifier, callback) {
    var object = void 0;

    if (objectOrIdentifier.isObjectExpression()) {
      object = objectOrIdentifier;
    }

    if (!object) {
      var declarator = findRootDeclarator(objectOrIdentifier);
      if (declarator && declarator.get('init').isObjectExpression()) {
        object = declarator.get('init');
      }
    }

    if (object) {
      var properties = object.get('properties');
      for (var i = 0; i < properties.length; i++) {
        var property = properties[i];
        callback(property);
      }
    }
  }

  function annotateFunction(path) {
    return annotateFunctionImpl(path, path);
  }

  function annotateObjectProperties(object) {
    if (!object.isObjectExpression()) {
      return;
    }
    var properties = object.get('properties');
    for (var i = 0; i < properties.length; i++) {
      var property = properties[i];
      annotateFunction(property.get('value'));
    }
  }

  function last(array) {
    return array[array.length - 1];
  }

  function annotateModuleConfigFunction(path) {
    var moduleLastArg = last(path.parentPath.get('arguments'));
    annotateFunction(moduleLastArg);
    moduleLastArg.traverse(configVisitor);
    moduleLastArg.stop();
  }

  function annotateModuleType(annotationCandidate) {
    var match = TYPES.test(getTypeName(annotationCandidate));

    if (match) {
      var candidate = last(annotationCandidate.get('arguments'));

      annotateFunction(candidate);

      switch (getTypeName(annotationCandidate)) {
        case 'provider':
          candidate.traverse(providerGetVisitor);
          break;
        case 'directive':
        case 'component':
          candidate.traverse(directiveControllerVisitor);
          break;
        case 'config':
          candidate.traverse(configVisitor);
          break;
      }

      candidate.traverse(defaultVisitor);
      candidate.stop();
    }
  }

  return {
    visitor: {
      Program: function Program(path, _ref2) {
        var opts = _ref2.opts;

        configuration = Array.isArray(opts) ? opts : [];
      },
      CallExpression: function CallExpression(path) {
        if (isAngularModule(path)) {
          annotateModuleType(path);
        }
      }
    }
  };
};